@@ -16,7 +16,25 @@
 import shlex
 import string
 
-from magnetodb.common import setup_global_env
+from oslo_context import context
+
+from magnetodb.api.openstack.v1 import utils
+from magnetodb import common as mdb_common
+from magnetodb.openstack.common import log as logging
+from magnetodb import policy
+
+LOG = logging.getLogger(__name__)
+
+
+def enforce_policy(rule):
+    def decorator(f):
+        def wrapper(req, project_id, *args, **kwargs):
+            utils.check_project_id(project_id)
+            policy.enforce(context.get_current(), rule, {})
+            LOG.debug('RBAC: Authorization granted')
+            return f(req, project_id, *args, **kwargs)
+        return wrapper
+    return decorator
 
 
 def with_global_env(default_program=None):
@@ -27,7 +45,7 @@ def with_global_env(default_program=None):
             program = options.get("program", default_program)
             s = string.Template(oslo_config_args)
             oslo_config_args = shlex.split(s.substitute(**options))
-            setup_global_env(program, oslo_config_args)
+            mdb_common.setup_global_env(program, oslo_config_args)
             return f(global_conf, **local_conf)
         return wrapped
     return decorator
