@@ -13,15 +13,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from magnetodb.api.amz.dynamodb.action import DynamoDBAction
-from magnetodb.api.amz.dynamodb import parser
-
+from magnetodb.api.amz.dynamodb import action
+from magnetodb.api.amz import parser
+from magnetodb.api.amz import exception as amz_exception
 from magnetodb import storage
-from magnetodb.common import exception
 from magnetodb.storage import models
 
 
-class QueryDynamoDBAction(DynamoDBAction):
+class QueryDynamoDBAction(action.DynamoDBAction):
     schema = {
         "required": [parser.Props.KEY_CONDITIONS,
                      parser.Props.TABLE_NAME],
@@ -38,12 +37,7 @@ class QueryDynamoDBAction(DynamoDBAction):
                 "type": "boolean"
             },
 
-            parser.Props.EXCLUSIVE_START_KEY: {
-                "type": "object",
-                "patternProperties": {
-                    parser.ATTRIBUTE_NAME_PATTERN: parser.Types.ITEM_VALUE
-                }
-            },
+            parser.Props.EXCLUSIVE_START_KEY: parser.Types.ITEM,
 
             parser.Props.INDEX_NAME: {
                 "type": "string",
@@ -58,7 +52,7 @@ class QueryDynamoDBAction(DynamoDBAction):
                         "properties": {
                             parser.Props.ATTRIBUTE_VALUE_LIST: {
                                 "type": "array",
-                                "items": parser.Types.ITEM_VALUE
+                                "items": parser.Types.TYPED_ATTRIBUTE_VALUE
                             },
                             parser.Props.COMPARISON_OPERATOR: (
                                 parser.Types.QUERY_OPERATOR
@@ -144,12 +138,12 @@ class QueryDynamoDBAction(DynamoDBAction):
                 models.ORDER_TYPE_DESC
             )
         except Exception:
-            raise exception.ValidationException()
+            raise amz_exception.AWSValidationException()
 
         try:
             # select item
-            result = storage.select_item(
-                self.context, table_name, indexed_condition_map,
+            result = storage.query(
+                self.tenant, table_name, indexed_condition_map,
                 select_type=select_type, index_name=index_name, limit=limit,
                 consistent=consistent_read, order_type=order_type,
                 exclusive_start_key=exclusive_start_key_attributes
@@ -183,7 +177,5 @@ class QueryDynamoDBAction(DynamoDBAction):
                         result.last_evaluated_key)
                 )
             return response
-        except exception.AWSErrorResponseException as e:
-            raise e
         except Exception:
-            raise exception.AWSErrorResponseException()
+            raise amz_exception.AWSErrorResponseException()
