@@ -13,16 +13,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from magnetodb.api.amz.dynamodb.action import DynamoDBAction
-from magnetodb.api.amz.dynamodb import parser
-
+from magnetodb.api.amz.dynamodb import action
+from magnetodb.api.amz import exception as amz_exception
+from magnetodb.api.amz import parser
 from magnetodb import storage
 from magnetodb.storage import models
 
-from magnetodb.common import exception
 
-
-class UpdateItemDynamoDBAction(DynamoDBAction):
+class UpdateItemDynamoDBAction(action.DynamoDBAction):
     schema = {
         "required": [parser.Props.KEY,
                      parser.Props.TABLE_NAME],
@@ -36,7 +34,9 @@ class UpdateItemDynamoDBAction(DynamoDBAction):
                             parser.Props.EXISTS: {
                                 "type": "boolean",
                             },
-                            parser.Props.VALUE: parser.Types.ITEM_VALUE
+                            parser.Props.VALUE: (
+                                parser.Types.TYPED_ATTRIBUTE_VALUE
+                            )
                         }
                     }
                 }
@@ -50,18 +50,15 @@ class UpdateItemDynamoDBAction(DynamoDBAction):
                         "required": [parser.Props.ACTION],
                         "properties": {
                             parser.Props.ACTION: parser.Types.ACTION_TYPE,
-                            parser.Props.VALUE: parser.Types.ITEM_VALUE
+                            parser.Props.VALUE: (
+                                parser.Types.TYPED_ATTRIBUTE_VALUE
+                            )
                         }
                     }
                 }
             },
 
-            parser.Props.KEY: {
-                "type": "object",
-                "patternProperties": {
-                    parser.ATTRIBUTE_NAME_PATTERN: parser.Types.ITEM_VALUE
-                }
-            },
+            parser.Props.KEY: parser.Types.ITEM,
 
             parser.Props.RETURN_CONSUMED_CAPACITY: (
                 parser.Types.RETURN_CONSUMED_CAPACITY
@@ -72,10 +69,7 @@ class UpdateItemDynamoDBAction(DynamoDBAction):
             parser.Props.RETURN_VALUES: {
                 "type": "string",
                 "enum": [parser.Values.RETURN_VALUES_NONE,
-                         parser.Values.RETURN_VALUES_ALL_OLD,
-                         parser.Values.RETURN_VALUES_ALL_NEW,
-                         parser.Values.RETURN_VALUES_UPDATED_OLD,
-                         parser.Values.RETURN_VALUES_UPDATED_NEW]
+                         parser.Values.RETURN_VALUES_ALL_OLD]
             },
             parser.Props.TABLE_NAME: parser.Types.TABLE_NAME
         }
@@ -117,49 +111,27 @@ class UpdateItemDynamoDBAction(DynamoDBAction):
                 parser.Props.RETURN_CONSUMED_CAPACITY,
                 parser.Values.RETURN_CONSUMED_CAPACITY_NONE
             )
-
-            select_result = None
-
-            indexed_condition_map_for_select = {
-                name: models.IndexedCondition.eq(value)
-                for name, value in key_attributes.iteritems()
-            }
         except Exception:
-            raise exception.ValidationException()
+            raise amz_exception.AWSValidationException()
 
         try:
-            if return_values in (parser.Values.RETURN_VALUES_UPDATED_OLD,
-                                 parser.Values.RETURN_VALUES_ALL_OLD):
-
-                select_result = storage.select_item(
-                    self.context, table_name,
-                    indexed_condition_map_for_select)
-
             # update item
-            result = storage.update_item(
-                self.context,
+            result, old_item = storage.update_item(
+                self.tenant,
                 table_name,
                 key_attribute_map=key_attributes,
                 attribute_action_map=attribute_updates,
                 expected_condition_map=expected_item_conditions)
 
             if not result:
-                raise exception.AWSErrorResponseException()
-
-            if return_values in (parser.Values.RETURN_VALUES_UPDATED_NEW,
-                                 parser.Values.RETURN_VALUES_ALL_NEW):
-
-                select_result = storage.select_item(
-                    self.context, table_name,
-                    indexed_condition_map_for_select)
+                raise amz_exception.AWSErrorResponseException()
 
             # format response
             response = {}
 
             if return_values != parser.Values.RETURN_VALUES_NONE:
                 response[parser.Props.ATTRIBUTES] = (
-                    parser.Parser.format_item_attributes(
-                        select_result.items[0])
+                    parser.Parser.format_item_attributes(old_item)
                 )
 
             if (return_item_collection_metrics !=
@@ -167,8 +139,7 @@ class UpdateItemDynamoDBAction(DynamoDBAction):
                 response[parser.Props.ITEM_COLLECTION_METRICS] = {
                     parser.Props.ITEM_COLLECTION_KEY: {
                         parser.Parser.format_item_attributes(
-                            models.AttributeValue(models.ATTRIBUTE_TYPE_STRING,
-                                                  "key")
+                            models.AttributeValue("S", "key")
                         )
                     },
                     parser.Props.SIZE_ESTIMATED_RANGE_GB: [0]
@@ -183,7 +154,7 @@ class UpdateItemDynamoDBAction(DynamoDBAction):
                 )
 
             return response
-        except exception.AWSErrorResponseException as e:
+        except amz_exception.AWSErrorResponseException as e:
             raise e
         except Exception:
-            raise exception.AWSErrorResponseException()
+            raise amz_exception.AWSErrorResponseException()
