@@ -13,11 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from magnetodb.common import wsgi
+from oslo_middleware import base as wsgi
 
 from magnetodb.openstack.common import log as logging
 
-logger = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 HTTP_CONNECTION = "Connection"
 
