@@ -15,25 +15,16 @@
 #    under the License.
 
 import hashlib
-
 import requests
-import webob
 
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_serialization import jsonutils as json
 
-from magnetodb.api.amz.dynamodb.exception import AWSAccessDeniedError
-from magnetodb.api.amz.dynamodb.exception import AWSInvalidClientTokenIdError
-from magnetodb.api.amz.dynamodb.exception import AWSErrorResponseException
-from magnetodb.api.amz.dynamodb.exception import AWSSignatureError
-from magnetodb.api.amz.dynamodb.exception import AWSIncompleteSignatureError
-
-from magnetodb.common import wsgi
-
-from magnetodb.openstack.common import jsonutils as json
+from magnetodb.api.amz import exception as amz_exception
+from oslo_middleware import base as wsgi
+from magnetodb.i18n import _, _LE, _LI
 from magnetodb.openstack.common import log as logging
-from magnetodb.openstack.common import gettextutils
-from magnetodb.openstack.common.gettextutils import _
-gettextutils.install('magnetodb')
+
 
 LOG = logging.getLogger(__name__)
 
@@ -59,7 +50,7 @@ class EC2Token(wsgi.Middleware):
 
     def __init__(self, app, conf):
         self.conf = conf
-        self.application = app
+        super(EC2Token, self).__init__(app)
 
     def _conf_get(self, name):
         return self.conf.get(name, None)
@@ -109,8 +100,7 @@ class EC2Token(wsgi.Middleware):
 
         return access
 
-    @webob.dec.wsgify(RequestClass=wsgi.Request)
-    def __call__(self, req):
+    def process_request(self, req):
         if not self._conf_get('multi_cloud'):
             return self._authorize(req, self._conf_get_auth_uri())
         else:
@@ -124,10 +114,10 @@ class EC2Token(wsgi.Middleware):
                 try:
                     LOG.debug("Attempt authorize on %s", auth_uri)
                     return self._authorize(req, auth_uri)
-                except AWSErrorResponseException as e:
+                except amz_exception.AWSErrorResponseException as e:
                     LOG.debug("Authorize failed: %s", e.__class__)
                     last_failure = e
-            raise last_failure or AWSAccessDeniedError()
+            raise last_failure or amz_exception.AWSAccessDeniedError()
 
     def _is_v2_token(self, token_body):
         return 'access' in token_body
@@ -141,7 +131,7 @@ class EC2Token(wsgi.Middleware):
         # here so that we can use both authentication methods.
         # Returning here just means the user didn't supply AWS
         # authentication and we'll let the app try native keystone next.
-        LOG.info(_("Checking AWS credentials.."))
+        LOG.info(_LI("Checking AWS credentials.."))
 
         signature = self._get_signature(req)
         if not signature:
@@ -150,23 +140,25 @@ class EC2Token(wsgi.Middleware):
             elif 'X-Auth-Token' in req.headers:
                 return self.application
             else:
-                LOG.info(_("No AWS Signature found."))
-                raise AWSIncompleteSignatureError()
+                LOG.info(_LI("No AWS Signature found."))
+                raise amz_exception.AWSIncompleteSignatureError()
 
         access = self._get_access(req)
         if not access:
             if 'X-Auth-User' in req.headers:
                 return self.application
             else:
-                LOG.info(_("No AWSAccessKeyId/Authorization Credential"))
-                raise AWSIncompleteSignatureError()
+                LOG.info(_LI("No AWSAccessKeyId/Authorization Credential"))
+                raise amz_exception.AWSIncompleteSignatureError()
 
-        LOG.info(_("AWS credentials found, checking against keystone."))
+        LOG.info(_LI("AWS credentials found, checking against keystone."))
 
         if not auth_uri:
-            LOG.error(_("Ec2Token authorization failed, no auth_uri "
-                        "specified in config file"))
-            raise AWSErrorResponseException(_('Service misconfigured'))
+            LOG.error(_LE("Ec2Token authorization failed, no auth_uri "
+                          "specified in config file"))
+            raise amz_exception.AWSErrorResponseException(
+                _LI('Service misconfigured')
+            )
         # Make a copy of args for authentication and signature verification.
         auth_params = dict(req.params)
         # 'Signature' param Not part of authentication args
@@ -188,7 +180,7 @@ class EC2Token(wsgi.Middleware):
         headers = {'Content-Type': 'application/json'}
 
         keystone_ec2_uri = self._conf_get_keystone_ec2_uri(auth_uri)
-        LOG.info(_('Authenticating with %s'), keystone_ec2_uri)
+        LOG.info(_LI('Authenticating with %s'), keystone_ec2_uri)
         response = requests.post(keystone_ec2_uri, data=creds_json,
                                  headers=headers)
         result = response.json()
@@ -206,10 +198,10 @@ class EC2Token(wsgi.Middleware):
                 metadata = result['token']['roles']
                 roles = [r['name'] for r in metadata]
             else:
-                raise AWSInvalidClientTokenIdError()
-            LOG.info(_("AWS authentication successful."))
+                raise amz_exception.AWSInvalidClientTokenIdError()
+            LOG.info(_LI("AWS authentication successful."))
         except (AttributeError, KeyError):
-            LOG.info(_("AWS authentication failure."))
+            LOG.info(_LI("AWS authentication failure."))
             # Try to extract the reason for failure so we can return the
             # appropriate AWS error via raising an exception
             try:
@@ -218,11 +210,11 @@ class EC2Token(wsgi.Middleware):
                 reason = None
 
             if reason == "EC2 access key not found.":
-                raise AWSInvalidClientTokenIdError()
+                raise amz_exception.AWSInvalidClientTokenIdError()
             elif reason == "EC2 signature not supplied.":
-                raise AWSSignatureError()
+                raise amz_exception.AWSSignatureError()
             else:
-                raise AWSAccessDeniedError()
+                raise amz_exception.AWSAccessDeniedError()
 
         # Authenticated!
         ec2_creds = {'ec2Credentials': {'access': access,
