@@ -1,6 +1,6 @@
-# -*- encoding: utf-8 -*-
-#
-# Copyright © 2013 Unitedstack Inc.
+# Copyright 2014 Mirantis Inc.
+# Copyright 2014 Symantec Corporation
+# Copyright 2013 Unitedstack Inc.
 #
 # Author: Jianing YANG (jianingy@unitedstack.com)
 #
@@ -24,44 +24,29 @@ import traceback
 
 import webob
 
-from magnetodb.common import wsgi
+from magnetodb.common import exception
+from oslo_middleware import base as wsgi
 
 from magnetodb.openstack.common import log as logging
 
-logger = logging.getLogger(__name__)
 
-
-class Fault(object):
-
-    def __init__(self, error):
-        self.error = error
-
-    @webob.dec.wsgify(RequestClass=wsgi.Request)
-    def __call__(self, req):
-        serializer = wsgi.ResponseSerializer()
-        resp = serializer.serialize(self.error, req.content_type)
-        default_webob_exc = webob.exc.HTTPInternalServerError()
-        resp.status_code = self.error.get('code', default_webob_exc.code)
-        return resp
+LOG = logging.getLogger(__name__)
 
 
 class FaultWrapper(wsgi.Middleware):
     """Replace error body with something the client can parse."""
 
+    DEFAULT_WEBOB_EXCEPTION = webob.exc.HTTPInternalServerError()
+
     error_map = {
         # Common errors
-        'InternalFailure': webob.exc.HTTPInternalServerError,
         'RequestQuotaExceeded': webob.exc.HTTPTooManyRequests,
-        'OverLimit': webob.exc.HTTPRequestEntityTooLarge,
-        'InvalidClientToken': webob.exc.HTTPUnauthorized,
-        'Forbidden': webob.exc.HTTPForbidden,
-        'MalformedQueryString': webob.exc.HTTPNotFound,
         'ServiceUnavailable': webob.exc.HTTPServiceUnavailable,
-        'InvalidParameterCombination': webob.exc.HTTPBadRequest,
-        'InvalidParameterValue': webob.exc.HTTPBadRequest,
+        'Forbidden': webob.exc.HTTPForbidden,
         'InvalidQueryParameter': webob.exc.HTTPBadRequest,
         'ValidationError': webob.exc.HTTPBadRequest,
-        'MissingParameter': webob.exc.HTTPBadRequest,
+
+        'ResourceInUseException': webob.exc.HTTPBadRequest,
 
         # Table errors
         'TableAlreadyExistsException': webob.exc.HTTPBadRequest,
@@ -113,12 +98,29 @@ class FaultWrapper(wsgi.Middleware):
 
         return error
 
+    def _create_error_response(self, ex):
+        error_info = self._error(ex)
+
+        return webob.Response(
+            json_body=error_info,
+            status=error_info.get('code', self.DEFAULT_WEBOB_EXCEPTION.code)
+        )
+
     def process_request(self, req):
         try:
             return req.get_response(self.application)
+        except (exception.BackendInteractionException,
+                exception.ValidationError) as ex:
+            LOG.debug(ex)
+
+            return self._create_error_response(ex)
         except Exception as ex:
-            logger.exception(ex)
-            return req.get_response(Fault(self._error(ex)))
+            # some lower level exception. It is better to know about it
+            # so, log the original message
+            LOG.exception(ex)
+            # but don't propagate internal details beyond here
+            ex.args = (u'message="An Internal Error Occurred"',)
+            return self._create_error_response(ex)
 
     @classmethod
     def factory_method(cls, global_config, **local_config):
