@@ -1,5 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
+# Copyright 2014 Mirantis Inc.
 # Copyright 2011 OpenStack Foundation.
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
@@ -38,12 +37,12 @@ import os
 import sys
 import traceback
 
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_serialization import jsonutils as json
 from six import moves
 
-from magnetodb.openstack.common.gettextutils import _  # noqa
+from magnetodb.i18n import _, _LC, _LW
 from magnetodb.openstack.common import importutils
-from magnetodb.openstack.common import jsonutils
 from magnetodb.openstack.common import local
 
 
@@ -241,12 +240,12 @@ class ContextAdapter(BaseLoggerAdapter):
         return self.logger.handlers
 
     def deprecated(self, msg, *args, **kwargs):
-        stdmsg = _("Deprecated: %s") % msg
+        stdmsg = "Deprecated: %s"
         if CONF.fatal_deprecations:
-            self.critical(stdmsg, *args, **kwargs)
-            raise DeprecatedConfig(msg=stdmsg)
+            self.critical(_LC(stdmsg) % msg, *args, **kwargs)
+            raise DeprecatedConfig(msg=_(stdmsg))
         else:
-            self.warn(stdmsg, *args, **kwargs)
+            self.warn(_LW(stdmsg) % msg, *args, **kwargs)
 
     def process(self, msg, kwargs):
         if 'extra' not in kwargs:
@@ -319,7 +318,7 @@ class JSONFormatter(logging.Formatter):
         if record.exc_info:
             message['traceback'] = self.formatException(record.exc_info)
 
-        return jsonutils.dumps(message)
+        return json.dumps(message)
 
 
 def _create_logging_excepthook(product_name):
