@@ -15,45 +15,44 @@
 #    under the License.
 
 import collections
-from copy import copy
-from datetime import datetime
-
-from threading import Lock
-
-from cassandra import encoder
+import copy
+from oslo_utils import timeutils
+import threading
 
 from magnetodb.common import exception
 from magnetodb.common import probe
 from magnetodb.storage import models
-from magnetodb.storage.table_info_repo import TableInfo
-from magnetodb.storage.table_info_repo import TableInfoRepository
+from magnetodb.storage import table_info_repo
+
+from cassandra import encoder
 
 
-class CassandraTableInfoRepository(TableInfoRepository):
+class CassandraTableInfoRepository(table_info_repo.TableInfoRepository):
     SYSTEM_TABLE_TABLE_INFO = 'magnetodb.table_info'
-    __field_list = ("schema", "internal_name", "status", "last_updated")
+    __field_list = ("id", "schema", "internal_name", "status",
+                    "last_update_date_time", "creation_date_time")
     __creating_to_active_field_list_to_update = ("internal_name", "status")
 
-    def _save_table_info_to_cache(self, context, table_info):
-        tenant_tables_cache = self.__table_info_cache.get(context.tenant)
+    def _save_table_info_to_cache(self, tenant, table_info):
+        tenant_tables_cache = self.__table_info_cache.get(tenant)
         if tenant_tables_cache is None:
             tenant_tables_cache = {}
-            self.__table_info_cache[context.tenant] = tenant_tables_cache
+            self.__table_info_cache[tenant] = tenant_tables_cache
 
         tenant_tables_cache[table_info.name] = table_info
 
-    def _get_table_info_from_cache(self, context, table_name):
-        tenant_tables_cache = self.__table_info_cache.get(context.tenant)
+    def _get_table_info_from_cache(self, tenant, table_name):
+        tenant_tables_cache = self.__table_info_cache.get(tenant)
         if tenant_tables_cache is None:
             return None
         table_info_cached = tenant_tables_cache.get(table_name)
         if table_info_cached is None:
             return table_info_cached
 
-        return copy(table_info_cached)
+        return copy.copy(table_info_cached)
 
-    def _remove_table_info_from_cache(self, context, table_name):
-        tenant_tables_cache = self.__table_info_cache.get(context.tenant)
+    def _remove_table_info_from_cache(self, tenant, table_name):
+        tenant_tables_cache = self.__table_info_cache.get(tenant)
         if tenant_tables_cache is None:
             return None
 
@@ -62,18 +61,20 @@ class CassandraTableInfoRepository(TableInfoRepository):
     def __init__(self, cluster_handler):
         self.__cluster_handler = cluster_handler
         self.__table_info_cache = {}
-        self.__table_cache_lock = Lock()
+        self.__table_cache_lock = threading.Lock()
 
-    def get(self, context, table_name, fields_to_refresh=tuple()):
-        table_info = self._get_table_info_from_cache(context, table_name)
+    def get(self, tenant, table_name, fields_to_refresh=tuple()):
+        table_info = self._get_table_info_from_cache(tenant, table_name)
         if table_info is None:
             with self.__table_cache_lock:
-                table_info = self._get_table_info_from_cache(context,
+                table_info = self._get_table_info_from_cache(tenant,
                                                              table_name)
                 if table_info is None:
-                    table_info = TableInfo(table_name, None, None)
-                    self.__refresh(context, table_info)
-                    self._save_table_info_to_cache(context, table_info)
+                    table_info = table_info_repo.TableInfo(
+                        table_name, None, None, None
+                    )
+                    self.__refresh(tenant, table_info)
+                    self._save_table_info_to_cache(tenant, table_info)
                     return table_info
 
         if table_info.status == models.TableMeta.TABLE_STATUS_CREATING:
@@ -82,17 +83,16 @@ class CassandraTableInfoRepository(TableInfoRepository):
                 self.__creating_to_active_field_list_to_update
             )
         if fields_to_refresh:
-            self.__refresh(context, table_info, fields_to_refresh)
+            self.__refresh(tenant, table_info, fields_to_refresh)
 
         return table_info
 
     @probe.Probe(__name__)
-    def get_tenant_table_names(self, context, exclusive_start_table_name=None,
-                               limit=None):
+    def list_tables(self, tenant, exclusive_start_table_name=None, limit=None):
         query_builder = collections.deque()
         query_builder.append(
             "SELECT name FROM {} WHERE tenant='{}'".format(
-                self.SYSTEM_TABLE_TABLE_INFO, context.tenant
+                self.SYSTEM_TABLE_TABLE_INFO, tenant
             )
         )
 
@@ -110,7 +110,53 @@ class CassandraTableInfoRepository(TableInfoRepository):
 
         return [row['name'] for row in tables]
 
-    def __refresh(self, context, table_info, field_list=__field_list):
+    def _list_all_tables_no_paging(self, limit):
+        query_builder = collections.deque()
+        query_builder.append(
+            "SELECT tenant, name, status FROM {}".format(
+                self.SYSTEM_TABLE_TABLE_INFO)
+        )
+
+        if limit:
+            query_builder.append(" LIMIT {}".format(limit))
+
+        return self.__cluster_handler.execute_query("".join(query_builder),
+                                                    consistent=True)
+
+    @probe.Probe(__name__)
+    def list_all_tables(self, last_evaluated_tenant=None,
+                        last_evaluated_table=None, limit=None):
+        if last_evaluated_tenant is None:
+            return self._list_all_tables_no_paging(limit)
+
+        query_builder = collections.deque()
+        query_builder.append(
+            "SELECT tenant, name, status FROM {} "
+            "WHERE tenant = '{}'".format(
+                self.SYSTEM_TABLE_TABLE_INFO, last_evaluated_tenant)
+        )
+
+        if last_evaluated_table:
+            query_builder.append(
+                " AND name = '{}'".format(last_evaluated_table)
+            )
+
+        head = self.__cluster_handler.execute_query("".join(query_builder),
+                                                    consistent=True)
+
+        if limit is None or len(head) >= limit:
+            return head
+
+        query = ("SELECT tenant, name, status FROM {} WHERE "
+                 "TOKEN(tenant) > TOKEN('{}') LIMIT {}".format(
+                     self.SYSTEM_TABLE_TABLE_INFO, last_evaluated_tenant,
+                     limit - len(head)))
+
+        tail = self.__cluster_handler.execute_query(query, consistent=True)
+
+        return head + tail
+
+    def __refresh(self, tenant, table_info, field_list=__field_list):
         query_builder = collections.deque()
         query_builder.append("SELECT ")
         query_builder.append(",".join(map('"{}"'.format, field_list)))
@@ -118,7 +164,7 @@ class CassandraTableInfoRepository(TableInfoRepository):
         query_builder.append(
             " FROM {} WHERE tenant='{}' AND name='{}'".format(
                 self.SYSTEM_TABLE_TABLE_INFO,
-                context.tenant, table_info.name
+                tenant, table_info.name
             )
         )
 
@@ -127,7 +173,7 @@ class CassandraTableInfoRepository(TableInfoRepository):
         )
 
         if not result:
-            self._remove_table_info_from_cache(context, table_info.name)
+            self._remove_table_info_from_cache(tenant, table_info.name)
             raise exception.TableNotExistsException(
                 "Table '{}' does not exist".format(table_info.name)
             )
@@ -137,11 +183,13 @@ class CassandraTableInfoRepository(TableInfoRepository):
             setattr(table_info, name, value)
         return True
 
-    def update(self, context, table_info, field_list=None):
+    def update(self, tenant, table_info, field_list=None):
         if not field_list:
             field_list = self.__field_list
 
-        table_info.last_updated = datetime.now()
+        if 'last_update_date_time' not in field_list:
+            field_list.append('last_update_date_time')
+        table_info.last_update_date_time = timeutils.utcnow()
 
         enc = encoder.Encoder()
 
@@ -163,7 +211,7 @@ class CassandraTableInfoRepository(TableInfoRepository):
 
         query_builder.append(
             " WHERE tenant='{}' AND name='{}' IF exists=1".format(
-                context.tenant, table_info.name
+                tenant, table_info.name
             )
         )
 
@@ -173,21 +221,22 @@ class CassandraTableInfoRepository(TableInfoRepository):
 
         if not result[0]['[applied]']:
             raise exception.TableNotExistsException(
-                "Table {} is not exists".format(table_info.name)
+                "Table {} does not exist".format(table_info.name)
             )
-        self._remove_table_info_from_cache(context, table_info.name)
+        self._remove_table_info_from_cache(tenant, table_info.name)
         return True
 
-    def save(self, context, table_info):
+    def save(self, tenant, table_info):
         enc = encoder.Encoder()
 
         query_builder = collections.deque()
         query_builder.append(
             'INSERT INTO {} '
-            '(exists, tenant, name, "schema", status,'
-            ' internal_name, last_updated)'
-            "VALUES(1, '{}', '{}'".format(
-                self.SYSTEM_TABLE_TABLE_INFO, context.tenant, table_info.name
+            '(exists, tenant, name, id, "schema", status,'
+            ' internal_name, last_update_date_time, creation_date_time)'
+            "VALUES(1, '{}', '{}', {}".format(
+                self.SYSTEM_TABLE_TABLE_INFO, tenant, table_info.name,
+                enc.cql_encode_all_types(table_info.id)
             )
         )
 
@@ -211,9 +260,13 @@ class CassandraTableInfoRepository(TableInfoRepository):
         else:
             query_builder.append(",null")
 
-        table_info.last_updated = datetime.now()
+        table_info.last_update_date_time = timeutils.utcnow()
         query_builder.append(", {}".format(
-            enc.cql_encode_datetime(table_info.last_updated)))
+            enc.cql_encode_datetime(table_info.last_update_date_time)))
+
+        table_info.creation_date_time = table_info.last_update_date_time
+        query_builder.append(", {}".format(
+            enc.cql_encode_datetime(table_info.creation_date_time)))
 
         query_builder.append(") IF NOT EXISTS")
 
@@ -226,16 +279,16 @@ class CassandraTableInfoRepository(TableInfoRepository):
                 "Table {} already exists".format(table_info.name)
             )
 
-        self._save_table_info_to_cache(context, copy(table_info))
+        self._save_table_info_to_cache(tenant, copy.copy(table_info))
         return True
 
-    def delete(self, context, table_name):
+    def delete(self, tenant, table_name):
         query = (
             "DELETE FROM {}"
             " WHERE tenant='{}' AND name='{}'".format(
-                self.SYSTEM_TABLE_TABLE_INFO, context.tenant, table_name
+                self.SYSTEM_TABLE_TABLE_INFO, tenant, table_name
             )
         )
         self.__cluster_handler.execute_query(query, consistent=True)
-        self._remove_table_info_from_cache(context, table_name)
+        self._remove_table_info_from_cache(tenant, table_name)
         return True
