@@ -1,4 +1,5 @@
 # Copyright 2014 Mirantis Inc.
+# Copyright 2014 Symantec Corporation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,31 +15,25 @@
 #    under the License.
 
 import httplib
-import json
 import mock
 import unittest
 
-from magnetodb.tests.fake import magnetodb_api_fake
+from oslo_serialization import jsonutils as json
+
+from magnetodb.tests.unittests.api.openstack.v1 import test_base_testcase
 
 
-class PutitemTestCase(unittest.TestCase):
+class PutitemTestCase(test_base_testcase.APITestCase):
     """The test for v1 ReST API."""
 
-    @classmethod
-    def setUpClass(cls):
-        magnetodb_api_fake.run_fake_magnetodb_api()
-
-    @classmethod
-    def tearDownClass(cls):
-        magnetodb_api_fake.stop_fake_magnetodb_api()
-
     @mock.patch('magnetodb.storage.put_item')
     def test_put_item(self, mock_put_item):
         headers = {'Content-Type': 'application/json',
                    'Accept': 'application/json'}
+        mock_put_item.return_value = (True, None)
 
         conn = httplib.HTTPConnection('localhost:8080')
-        url = '/v1/fake_project_id/data/tables/the_table/put_item'
+        url = '/v1/data/default_tenant/tables/the_table/put_item'
         body = """
             {
                 "item": {
@@ -49,7 +44,7 @@ class PutitemTestCase(unittest.TestCase):
                         "SS": ["Update","Multiple items","HelpMe"]
                     },
                     "ForumName": {
-                        "S": "Amazon DynamoDB"
+                        "S": "MagnetoDB"
                     },
                     "Message": {
                         "S": "I want to update multiple items."
@@ -93,7 +88,7 @@ class PutitemTestCase(unittest.TestCase):
                    'Accept': 'application/json'}
 
         conn = httplib.HTTPConnection('localhost:8080')
-        url = '/v1/fake_project_id/data/tables/the_table/put_item'
+        url = '/v1/data/default_tenant/tables/the_table/put_item'
         body = """
             {
                 "item": {
@@ -104,7 +99,7 @@ class PutitemTestCase(unittest.TestCase):
                         "SS": ["Update","Multiple items","HelpMe"]
                     },
                     "ForumName": {
-                        "S": "Amazon DynamoDB"
+                        "S": "MagnetoDB"
                     },
                     "Message": {
                         "S": "I want to update multiple items."
@@ -141,7 +136,7 @@ class PutitemTestCase(unittest.TestCase):
         response_payload = json.loads(json_response)
 
         expected = {'attributes': {
-            'ForumName': {'S': u'Amazon DynamoDB'},
+            'ForumName': {'S': u'MagnetoDB'},
             'LastPostDateTime': {'S': '201303190422'},
             'LastPostedBy': {'S': 'fred@example.com'},
             'Message': {'S': 'I want to update multiple items.'},
