@@ -1,18 +1,16 @@
-#!/bin/bash
-
-if [ -z $1 ]
-then
-    CASSANDRA_AMOUNT_NODES=1
-else
-    CASSANDRA_AMOUNT_NODES=$1
-fi
+#!/bin/bash -x
 
+CASSANDRA_AMOUNT_NODES=1
+MAGNETODB_DIR=$WORKSPACE
 CCM_REPO=${CCM_REPO:-'https://github.com/pcmanus/ccm.git'}
 CCM_BRANCH=${CCM_BRANCH:-master}
 CCM_DIR=${CCM_DIR:-$HOME/ccm}
-CASSANDRA_VER=${CASSANDRA_VER:-2.0.8}
+CASSANDRA_VER=${CASSANDRA_VER:-2.1.3}
 CASSANDRA_CLUSTER_NAME=${CASSANDRA_CLUSTER_NAME:-test}
 CASSANDRA_REPL_FACTOR=${CASSANDRA_REPL_FACTOR:-1}
+GRADLE_VER=${GRADLE_VER:-2.2.1}
+GRADLE_REPO=${GRADLE_REPO:-"https://services.gradle.org/distributions/gradle-$GRADLE_VER-bin.zip"}
+GRADLE_DEST=`mktemp -d`
 
 function fix_etc_hosts {
     # HPcloud stopped adding the hostname to /etc/hosts with their
@@ -47,7 +45,20 @@ function configure_cassandra() {
         let addr=$addr+1
         let n=$n+1
     done
+
     ccm status $CASSANDRA_CLUSTER_NAME || ccm create $CASSANDRA_CLUSTER_NAME -v $CASSANDRA_VER
+
+    # Build cassandra custom index
+    wget -q $GRADLE_REPO -O $GRADLE_DEST/gradle.zip
+    cd $GRADLE_DEST
+    unzip gradle.zip
+    PATH=$PATH:$GRADLE_DEST/gradle-$GRADLE_VER/bin
+    cd $MAGNETODB_DIR/contrib/cassandra/magnetodb-cassandra-custom-indices
+    gradle build
+    CCIV=`grep '^version' build.gradle | cut -d"'" -f2`
+    cp $MAGNETODB_DIR/contrib/cassandra/magnetodb-cassandra-custom-indices/build/libs/magnetodb-cassandra-custom-indices-$CCIV.jar ~/.ccm/repository/${CASSANDRA_VER}/lib/
+
+    # Populate cassandra nodes
     ccm populate -n $CASSANDRA_AMOUNT_NODES || true
 }
 
@@ -76,7 +87,10 @@ ccm start
 # create keyspace and table
 create_keyspace_cassandra magnetodb
 create_keyspace_cassandra user_default_tenant
-echo 'CREATE TABLE magnetodb.table_info(tenant text, name text, exists int, "schema" text, status text, internal_name text, last_updated timestamp, PRIMARY KEY(tenant,name));' >> ~/.ccm/cql.txt
+echo 'CREATE TABLE magnetodb.table_info(tenant text, name text, id uuid, exists int, "schema" text, status text, internal_name text, last_update_date_time timestamp, creation_date_time timestamp, PRIMARY KEY(tenant,name));' >> ~/.ccm/cql.txt
+echo 'CREATE TABLE magnetodb.backup_info(tenant text, table_name text, id uuid, name text, status text, start_date_time timestamp, finish_date_time timestamp, location text, strategy map<text, text>, PRIMARY KEY((tenant, table_name), id));' >> ~/.ccm/cql.txt
+echo 'CREATE TABLE magnetodb.restore_info(tenant text, table_name text, id uuid, status text, backup_id uuid, start_date_time timestamp, finish_date_time timestamp, source text, PRIMARY KEY((tenant, table_name), id));' >> ~/.ccm/cql.txt
+echo 'CREATE TABLE magnetodb.dummy(id int PRIMARY KEY);' >> ~/.ccm/cql.txt
 
 timeout 120 sh -c 'while ! nc -z 127.0.0.1 9160; do sleep 1; done' || echo 'Could not login at 127.0.0.1:9160'
 ccm node1 cqlsh -f ~/.ccm/cql.txt
